package com.cneeds.rvhelpers.adapter;


import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.view.ViewGroup;

import com.cneeds.rvhelpers.entity.IHeader;
import com.cneeds.rvhelpers.helpers.KeyEventHelper;
import com.cneeds.rvhelpers.viewholder.BaseViewHolder;
import com.cneeds.rvhelpers.viewholder.ContentHolder;
import com.cneeds.rvhelpers.viewholder.TitleHolder;

import java.util.ArrayList;
import java.util.List;

/**
 * 参考博客：https://www.jianshu.com/p/1558cbd20d17
 * https://blog.csdn.net/little762/article/details/79391807
 * https://blog.csdn.net/briblue/article/details/70211942
 * <p>
 * 参考github: https://github.com/tuacy/RecyclerPinnedHeader/blob/02ea221444a6b4810c49c158fb0930d87bd59906/app/src/main/java/com/tuacy/recyclerpinnedheader/grid/GridRecyclerAdapter.java
 * <p>
 * author : maiqh
 */
public class GridRecyclerAdapter<T extends Object> extends PinnedHeaderAdapter<BaseViewHolder> {

    protected static final int VIEW_TYPE_ITEM_HEADER = 0;
    protected static final int VIEW_TYPE_ITEM_CONTENT = 1;
    public static final int NONE_SELECT = -1;

    private List<T> mDataList;
    private int mHighListPosition = NONE_SELECT;
    private View mHighListItemView;
    private final KeyEventHelper mKeyEventHelper;
    private boolean mEditMode;
    private OnSelectListener mOnSelectListener;
    private OnItemClickListener mOnItemClickListener;

    /**
     * @param dataList
     * @param spanCount The number of columns in the grid
     */
    public GridRecyclerAdapter(List<T> dataList, int spanCount) {
        if (dataList == null) {
            mDataList = new ArrayList<>();
        } else {
            mDataList = dataList;
        }
        mKeyEventHelper = new KeyEventHelper(dataList, spanCount);
        registerAdapterDataObserver(new RecyclerView.AdapterDataObserver() {
            @Override
            public void onChanged() {
                super.onChanged();
                // 监听数据数据是否变化，如果有变化会及时更新坐标二维表
                mKeyEventHelper.resetCoordination(mDataList);
            }
        });
    }

    public void setDataList(List<T> dataList) {
        if (dataList == null) {
            mDataList = new ArrayList<>();
        } else {
            mDataList = dataList;
        }
        notifyDataSetChanged();
    }

    public List getDataList() {
        if (mDataList == null) {
            mDataList = new ArrayList<>();
        }
        return mDataList;
    }

    public int getAdapterPosition(int x, int y) {
        return mKeyEventHelper.getAdapterPosition(x, y);
    }

    /**
     * 移动到下一位置， 用于响应实体按键/旋钮的控制
     *
     * @param x 外界要求移动到的x轴位置
     * @param y 外界要求移动到的y轴位置
     * @return  返回的PositionBean 包含xy实际会移动到的位置，和对应成员的adapter位置
     */
    public KeyEventHelper.PositionBean moveToNext(int x, int y) {
        // 防止异常情况导致crash
        if (x < 0) x = 0;
        if (y < 0) y = 0;
        KeyEventHelper.PositionBean positionBean = mKeyEventHelper.moveToNext(x, y, mHighListPosition);
        if (positionBean.getAdapterPosition() >= 0) {
            setCurrentHighLight(positionBean.getAdapterPosition());
        }
        return positionBean;
    }

    /**
     * 获取当前高亮item的位置，注意：默认值是-1 GridRecyclerAdapter.NONE_SELECT
     * @return
     */
    public int getCurrentHighLightPosition() {
        return mHighListPosition;
    }

    public View getHighLightView() {
        if (mHighListPosition == NONE_SELECT) {
            mHighListItemView = null;
        }
        return mHighListItemView;
    }

    public void setHighListItemView(View view) {
        mHighListItemView = view;
    }

    /**
     * 设置高亮的item, 用于响应实体按键/旋钮的控制
     *
     * @param highListPosition 高亮位置
     * @return 高亮位置
     */
    public int setCurrentHighLight(int highListPosition) {

        if (highListPosition < 0) {
            mHighListPosition = NONE_SELECT;
            mHighListItemView = null;
        } else if (getDataList().size() > mHighListPosition) {
            T t = mDataList.get(highListPosition);
            if (t instanceof IHeader) {
                // 跳过标题
                if (mHighListPosition > highListPosition) {
                    mHighListPosition = highListPosition - 1;
                } else {
                    mHighListPosition = highListPosition + 1;
                }
            } else {
                mHighListPosition = highListPosition;
            }
        } else {
            mHighListPosition = mDataList.size() - 1;
        }
        notifyDataSetChanged();
        return mHighListPosition;
    }

    public boolean getEditMode() {
        return mEditMode;
    }

    /**
     * 把选中的类删除
     */
    public void deleteSelected() {

        List<T> selectedItems = getSelectedItems();

        // 删除选中的数据
        for (int i = 0; i < selectedItems.size(); i++) {
            T t1 = selectedItems.get(i);
            if (!getDataList().contains(t1)) {
                break;
            }
            int index = mDataList.indexOf(t1);
            boolean remove = mDataList.remove(t1);
            if (remove) {
                // 下面处理标题的删除，注意：下面的index对应的不再是被删除的数据
                if (mDataList.size() <= index) {
                    // 删除了最后一个
                    T t = mDataList.get(mDataList.size() - 1);
                    if (t instanceof IHeader) {
                        // 如果最后一个是标题，就把它删除
                        mDataList.remove(mDataList.size() - 1);
                    }
                } else if (index > 0) { // 第一个是标题，index理论上不会为0，这里加个判断预防万一
                    T t = mDataList.get(index);
                    T previousT = mDataList.get(index - 1);
                    // 如果连续两个都是标题，就把第一个标题删除
                    if (t instanceof IHeader && previousT instanceof IHeader) {
                        mDataList.remove(index - 1);
                    }
                }
            }
        }
        clearAllSelection();
    }

    /**
     * select all items
     */
    public void selectAll() {
        getSelectedItems().clear();
        for (int i = 0; i < getDataList().size(); i++) {
            T t = mDataList.get(i);
            if (!(t instanceof IHeader)) {
                getSelectedItems().add(t);
            }
        }
        notifyDataSetChanged();
    }

    /**
     * 设置编辑模式
     *
     * @param editMode true: 编辑模式， false：非编辑模式
     */
    public void setEditMode(boolean editMode) {
        mEditMode = editMode;
        if (!mEditMode) {
            clearAllSelection();
        }
        notifyDataSetChanged();
    }

    /**
     * 切换选中/未选中
     *
     * @param view     被点击的itemView
     * @param position 被点击的itemView的位置
     * @param contentBean 被点击的itemView的数据bean
     */
    public void toggleSelection(View view, int position, Object contentBean) {
        boolean isChecked = toggleSelection(position, contentBean);
        if (mOnSelectListener != null) {
            mOnSelectListener.onSelect(view, position, isChecked, getSelectedItems(), mKeyEventHelper.getTotalCount());
        }
    }

    public void clickItem(View view, int position) {
        if (mOnItemClickListener != null) {
            mOnItemClickListener.onItemClick(view, position);
        }
    }

    @Override
    public BaseViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (viewType == VIEW_TYPE_ITEM_HEADER) {
            return new TitleHolder(parent.getContext(), parent);
        } else {
            return new ContentHolder(parent.getContext(), parent);
        }
    }

    @Override
    public void onBindViewHolder(BaseViewHolder holder, int position) {
        holder.bindData(getDataList().get(position), position, GridRecyclerAdapter.this);
    }

    @Override
    public int getItemCount() {
        return mDataList == null ? 0 : mDataList.size();
    }

    @Override
    public int getItemViewType(int position) {
        if (getDataList().get(position) instanceof IHeader) {
            return VIEW_TYPE_ITEM_HEADER;
        } else {
            return VIEW_TYPE_ITEM_CONTENT;
        }
    }

    @Override
    public boolean isPinnedPosition(int position) {
        return getItemViewType(position) == VIEW_TYPE_ITEM_HEADER;
    }

    public interface OnSelectListener {
        /**
         * 选择回调
         *
         * @param v                 当前被点击的view
         * @param adapterPosition   当前被点击的位置
         * @param isChecked          true：选择， false：取消选择
         * @param allSelectedItems 已经勾选的item
         * @param contentItemCount  除标题item外，全部item总数
         */
        void onSelect(View v, int adapterPosition, boolean isChecked, List allSelectedItems, int contentItemCount);
    }

    public void setOnSelectListener(OnSelectListener onSelectListener) {
        mOnSelectListener = onSelectListener;
    }

    public interface OnItemClickListener {
        /**
         * 选择回调
         *
         * @param v                 当前被点击的view
         * @param adapterPosition   当前被点击的位置
         */
        void onItemClick(View v, int adapterPosition);
    }

    public void setOnItemClickListener(OnItemClickListener onSelectListener) {
        mOnItemClickListener = onSelectListener;
    }
}
